package Slapd::Backend::Cyrus::Cyradm;

=head1 NAME

Slapd::Backend::Cyrus::Cyradm - Cyradm::IMAP::Admin wrapper

=head1 DESCRIPTION

Wrapper for Cyradm::IMAP::Admin, used to reconnect to Cyrus,
if connection was broken.

=cut

use utf8;
use strict;
use warnings;
use base 'Cyrus::IMAP::Admin';

BEGIN {
	foreach my $method ( qw/ listmailbox createmailbox listaclmailbox setaclmailbox / )
	{
		no strict 'refs';
		my $sm = "SUPER::$method";
		my $pm = __PACKAGE__ . '::' . $method;
		*{$pm} = sub {
			my $self = shift;
			for( 0 .. 1 )
			{
				if( wantarray )
				{
					my @result = $self->$sm(@_);
					if( defined( my $error = $self->error() ) )
					{
						$error =~ s/^.*: //;
						return @result if $error;
					}
					else
					{
						return @result;
					}
				}
				else
				{
					my $result = $self->$sm(@_);
					if( defined( my $error = $self->error() ) )
					{
						$error =~ s/^.*: //;
						return $result if $error;
					}
					else
					{
						return $result;
					}
				}
				$self->{cyrus} = Cyrus::IMAP->new(@{$self->{newopts}})
					or warn "Cannot reconnect to cyrus";
				$self->{cyrus}->authenticate(@{$self->{authopts}})
					or warn "Cannot reathenticate to cyrus";
			}
			return;
		};
	}
}

sub new
{
	my $class = shift;
	my $self = $class->SUPER::new(@_);
	return unless $self;
	$self->{newopts} = \@_;
	return $self;
}

1;

=head1 SEE ALSO

L<Cyradm::IMAP::Admin>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
